import './App.css';

import React, { useEffect, useState, useContext } from 'react';


import StepOne, { ServiceList } from './Steps/StepOne';
import StepTwo from './Steps/StepTwo';
import StepThree from './Steps/StepThree';
import StepFour from './Steps/StepFour';
import StepFive from './Steps/StepFive';

import { LangContext, BookingContext } from './Context';

import { Modal, notification, ConfigProvider } from 'antd';

import testItems from './testItems.json';

import getScreenType from './ScreenType';
import { fetchJSON, apiPaths } from './fetch';


function App() {
  const [screenType, setScreen] = useState(getScreenType());

  //create useContext for lang
  const [langData, setLangData] = useState(null);
  const [step, setStep] = useState(0);
  const [serviceItems, setServiceItems] = useState(null);
  const [bookingData, setBookingData] = useState({ currentStep: 1, selectedService: null, selectedDate: null, selectedTime: null, selectedUser: null });

  const [modal, contextHolderModal] = Modal.useModal();
  const [notificationAPI, contextHolderNotification] = notification.useNotification();

  // fetch my lang.json
  useEffect(() => {
    // get url query parameters
    const urlParams = new URLSearchParams(window.location.search);
    const storeID = urlParams.get('id');
    console.log(storeID);

    fetch('lang.json?id=' + storeID)
      .then((response) => response.json())
      .then((_langData) => {
        setLangData(_langData);

        fetchJSON(apiPaths.services, {}, {}, { key: "services" }).then((response) => {
          let _services = response.services;
          let services = [];

          for (let i = 0; i < _services.length; i++) {
            let service = _services[i];
            if (service.activities !== undefined && service.activities.length > 0) {
              services.push(service);
            }
          }

          setServiceItems({ "activities": services, "users": response.users, "maxDate": response.maxDate, "storeName": response.storeName });
        }).catch((error) => {
          console.log(error);
          modal.error({
            title: _langData.modalError.title,
            content: _langData.modalError["404"],
            footer: (_, { OkBtn, CancelBtn }) => (
              <>

              </>
            ),
          });
        });
      });




  }, []);

  useEffect(() => {
    window.addEventListener('resize', () => {
      setScreen(getScreenType());
    });
  }, []);

  if (!langData) return null;

  console.log(bookingData);
  return (<ConfigProvider
    theme={{
      token: {
        fontFamily: "Roboto, sans-serif",
        colorPrimary: "#6878d6",
        colorInfo: "#6878d6",
      },
    }}
  >
    <LangContext.Provider value={langData}>
      <BookingContext.Provider value={[bookingData, setBookingData]}>

        <div className='StepsContainer'>
          {bookingData.currentStep === 5 ? <StepFive screenType={screenType}
            serviceItems={serviceItems}
            isDisabled={bookingData.selectedDate === null}>Step 4</StepFive> :
            bookingData.currentStep === 4 ? <StepFour screenType={screenType}
              notificationAPI={notificationAPI}
              serviceItems={serviceItems}
              isDisabled={bookingData.selectedDate === null}>Step 4</StepFour> :
              <>
                {(screenType >= 2 || (screenType === 1 && bookingData.currentStep <= 2) || (bookingData.currentStep === 1)) ? <StepOne key="stepOneScreen"
                  screenType={screenType}
                  serviceItems={serviceItems}
                  canSubmit={bookingData.selectedService !== null} /> : null}
                {(screenType >= 1 && bookingData.currentStep <= 3) || bookingData.currentStep === 2 ? <StepTwo key="stepTwoScreen"
                  screenType={screenType}
                  serviceItems={serviceItems}
                  isDisabled={bookingData.selectedService === null}
                  canSubmit={bookingData.selectedDate !== null} /> : null}
                {screenType === 2 || bookingData.currentStep === 3 ? <StepThree key="stepThreeScreen"
                  screenType={screenType}
                  serviceItems={serviceItems}
                  isDisabled={bookingData.selectedDate === null} /> : null}
              </>}
        </div>
        {contextHolderNotification}
        {contextHolderModal}

      </BookingContext.Provider>
    </LangContext.Provider>
  </ConfigProvider>

  );
}

export default App;
