import './styles.css';

import React, { useContext, Image } from 'react';
import { LangContext } from '../Context';

import { Button, Flex, Typography, Spin } from 'antd';
import config from '../config';




function StepBasic({ children, screenType, titleText, headerColor, bg, stepNumber, currentStep, maxSteps, isDisabled, isLoading, canSubmit, onClickNext, onClickBack, storeName }) {
    const lang = useContext(LangContext);

    let displayNextButton = false;
    let displayBackButton = false;
    let displayDsgvo = false;

    let displayLogo = false;
    let displayLogoBesideButtons = false;

    let displayStoreName = false;

    if (stepNumber === 1) {
        displayDsgvo = true;
    }

    if (stepNumber === 2) {
        displayStoreName = true;
    }

    if (stepNumber === 4) {
        displayNextButton = true;
        displayBackButton = true;

        displayLogo = true;
        displayStoreName = true;
    } else if (stepNumber === 5) {
        displayNextButton = false;
        displayBackButton = false;

        displayLogo = true;
        displayStoreName = true;
    } else if (screenType === 0) {
        displayStoreName = true;
        displayNextButton = true;
        displayLogo = true;

        if (stepNumber > 1)
            displayBackButton = true;
    } else if (screenType === 1 && stepNumber === 1) {
        displayStoreName = true;
    } else if (screenType === 1 && stepNumber === 2) {

        if (currentStep === 1) {
            displayStoreName = false;
            displayNextButton = true;

            displayLogo = true;
        }
        if (currentStep === 3)
            displayBackButton = true;
    } else if (screenType === 1 && stepNumber === 3) {
        displayNextButton = true;

        displayLogo = true;
    } else if (screenType === 2 && stepNumber === 3) {
        displayLogo = true;
    }


    displayLogoBesideButtons = displayLogo && (displayNextButton || displayBackButton);


    const isNextButtonDisabled = isDisabled || !canSubmit;



    const buttonDivider = displayLogoBesideButtons ? <img className={"poweredByImageSmall"} src={config.publicURL + "/poweredByLogo.svg"} alt="Powered by ZeitAdler" /> : <div style={{ width: '1rem' }}></div>;

    if (!storeName)
        displayStoreName = true;

    return (

        <div className={'StepBasicContainer' + (stepNumber <= 3 ? " StepBasicContainerResize" : "")} style={{ backgroundColor: bg }}>
            <div className='StepBasicHeader' style={{ backgroundColor: headerColor }}>
                <Typography.Title level={3} style={{ opacity: displayStoreName ? 1 : 0 }} >{displayStoreName ? storeName : "|"}</Typography.Title>
                <Typography.Title level={2} >{titleText}</Typography.Title>
                {stepNumber < 5 ? <Typography >{lang.Steps.StepProgress.replaceAll("{0}", stepNumber).replaceAll("{1}", maxSteps)}</Typography> : null}

            </div>
            <div className={'StepBasicContent'}>
                {isDisabled || isLoading ? <>
                    <div className='StepBasicContentDisabled'>{isLoading ? <Flex justify="center" align="center" style={{ height: "100%" }}><Spin /></Flex> : null}</div>
                    <div className='StepBasicContentInner'>{children}</div>
                </> : <div className='StepBasicContentInner'>{children}</div>}

            </div>
            {displayLogo && !displayLogoBesideButtons ? <img className={"poweredByImage"} src={config.publicURL + "/poweredByLogo.svg"} alt="Powered by ZeitAdler" /> : null}

            {displayBackButton || displayNextButton || displayDsgvo ? <div className='StepBasicFooter'>
                {displayDsgvo ? <Typography.Link target='_blank' href={config.dsgvo_url} >{lang.Steps.DSGVO}</Typography.Link> : null}
                {displayBackButton ? <Button type='primary' size='large' onClick={onClickBack}>{lang.Steps.StepBack}</Button> : null}
                {buttonDivider}
                {displayNextButton ? <Button type='primary' size='large' disabled={isNextButtonDisabled} onClick={onClickNext}>{lang.Steps.StepNext}</Button> : null}

            </div> : null}
        </div>
    );
}

export default StepBasic;
