import config from "../config";
import StepBasic from './StepBasic';

import ReCAPTCHA from "react-google-recaptcha";

import React, { useEffect, useState, useContext, useRef } from 'react';
import { LangContext, BookingContext } from '../Context';

import {
    Spin, Card, Button, Checkbox, Flex, Typography, Form,
    Input
} from 'antd';

import dayjs, { Dayjs } from 'dayjs';

import { fetchJSON, apiPaths } from "../fetch";
import TextArea from "antd/es/input/TextArea";

import * as EmailValidator from 'email-validator';

import FinalBooking from "../components/FinalBooking";

function StepFour({ screenType, serviceItems, canSubmit, isDisabled, notificationAPI }) {
    const [bookingData, setBookingData] = useContext(BookingContext);
    const lang = useContext(LangContext);

    const [form] = Form.useForm();
    const [formData, setFormData] = useState({});
    const dsgvoCheckbox = useRef(null);
    const scrollToDsgvoCheckbox = () => dsgvoCheckbox.current.scrollIntoView({ behavior: 'smooth' });

    const canSubmitForm = formData.Name !== undefined && formData.EMail !== undefined;

    const [isLoading, setIsLoading] = useState(false);


    const [recaptchaValue, setRecaptchaValue] = useState(undefined);

    let chosenUser = {};

    if (bookingData.chosenUser !== null) {
        const userid = bookingData.chosenUser;
        const username = serviceItems.users[userid].name;
        chosenUser = { id: userid, name: username };
    }

    console.log(process.env.REACT_APP_RECAPTCHA_SITE_KEY)


    useEffect(() => {
        if (recaptchaValue !== undefined) {

            form.validateFields(['captcha']);
        }
    }, [recaptchaValue]);


    return <> <StepBasic
        screenType={screenType}
        storeName={serviceItems?.storeName}
        titleText={lang.Steps.StepFour.Title}
        headerColor={"var(--primary)"}
        bg={"var(--background)"}
        stepNumber={4}
        maxSteps={config.maxSteps}
        canSubmit={true}
        isLoading={isLoading}


        onClickNext={() => {

            form.submit();

            return;
        }}
        onClickBack={() => {
            bookingData.currentStep = 3;
            setBookingData({ ...bookingData });
        }}>

        <Flex horizontal="true" wrap="wrap" justify="space-evenly" align="center" style={{ marginTop: 10 }}>

            <Typography.Title level={5}>{lang.Steps.StepFour.info}</Typography.Title>
            <div style={{ width: "100%", maxWidth: 400, flexGrow: 1 }}>

                <FinalBooking selectedUser={chosenUser} infoText={lang.Steps.StepFour.notConfirmed} infoColor={"danger"} />
                <Form
                    form={form}
                    labelCol={{ span: 6 }}
                    wrapperCol={{ flex: 1 }}
                    onSubmitCapture={(e) => {
                        e.preventDefault();

                        form.submit();
                    }}
                    onFinish={(formData) => {
                        console.log("onFinish", formData);
                        let name = formData.Name;

                        // remove all double spaces
                        while (name.indexOf("  ") !== -1) {
                            name = name.replaceAll("  ", " ");
                        }

                        // remove spaces at the beginning and end
                        name = name.trim();

                        setIsLoading(true);

                        fetchJSON(apiPaths.booking, {

                            activity_id: bookingData.selectedService.id,
                            time: bookingData.selectedTime.unix() * 1000,
                            name: formData.Name,
                            email: formData.EMail,
                            message: formData.Message !== undefined ? formData.Message : "",
                            captcha: recaptchaValue,
                            employee: bookingData.chosenUser,
                        }, {}, { key: "booking" }
                        ).then((response) => {
                            if (response.status === "success") {
                                bookingData.currentStep = 5;
                                setBookingData({ ...bookingData });
                            } else if (response.status === "already booked") {
                                notificationAPI["error"]({
                                    message: lang.Steps.StepFour.errorTitle,
                                    description: lang.Steps.StepFour.errorTooSlow,
                                    duration: 30,
                                });

                                bookingData.currentStep = 3;
                                bookingData.selectedTime = null;
                                setBookingData({ ...bookingData });
                            } else {
                                notificationAPI["error"]({
                                    message: lang.Steps.StepFour.errorTitle,
                                    description: lang.Steps.StepFour.errorTryAgain,
                                });
                            }


                        }).catch((error) => {
                            console.error(error);

                            notificationAPI["error"]({
                                message: lang.Steps.StepFour.errorTitle,
                                description: lang.Steps.StepFour.errorTryAgain,
                            });

                        }).finally(() => {
                            setIsLoading(false);
                        });

                        //bookingData.currentStep = 5;
                        //setBookingData({ ...bookingData });
                    }}
                    style={{ marginBottom: 60 }}
                    scrollToFirstError={{ behavior: 'smooth', block: 'center', focus: true }}

                    colon={false}
                    size="large"
                >
                    <Form.Item name="Name" label={lang.Steps.StepFour.name}
                        validateFirst
                        validateDebounce={1000}
                        rules={[{ required: true, message: lang.Steps.StepFour.pleaseFill },
                        { max: 30, message: lang.Steps.StepFour.inputTooLong.replaceAll("{0}", 30) },
                        { min: 3, message: lang.Steps.StepFour.inputTooShort.replaceAll("{0}", 3) },
                        {
                            validator: (_, value) => {
                                let _value = value.replaceAll(" ", "");

                                // Allow letters, spaces, and special characters like hyphens and apostrophes
                                const isValidName = /^[\p{L} \-'’]+$/u.test(_value);




                                if (!isValidName) {
                                    return Promise.reject(lang.Steps.StepFour.invalidName);
                                }

                                return Promise.resolve();
                            },
                        }]}
                    >
                        <Input placeholder={lang.Steps.StepFour.exampleName} autoComplete="name" field={"formData.Name?.value"} />
                    </Form.Item>

                    <Form.Item name="EMail" label={lang.Steps.StepFour.email}
                        validateDebounce={1000}

                        rules={[{
                            required: true, validator: (_, value) => {
                                if (value === undefined || value === "") {
                                    return Promise.reject(lang.Steps.StepFour.pleaseFill);
                                }

                                if (!EmailValidator.validate(value)) {
                                    return Promise.reject(lang.Steps.StepFour.invalidEmail);
                                }

                                return Promise.resolve();

                            }
                        }]}>
                        <Input type="email" placeholder={lang.Steps.StepFour.exampleEmail} autoComplete="email" />
                    </Form.Item>

                    <Form.Item name="Message" label={lang.Steps.StepFour.message}
                        rules={[{ max: 1500, message: lang.Steps.StepFour.inputTooLong.replaceAll("{0}", 1500) }]}>
                        <TextArea placeholder={lang.Steps.StepFour.exampleMessage} autoSize={{ minRows: 2 }} />
                    </Form.Item>

                    <Form.Item name="dsgvo" valuePropName="checked"
                        rules={[
                            {
                                validator: (_, checked) =>
                                    checked ? Promise.resolve() : Promise.reject(new Error(lang.Steps.StepFour.dsgvoError)),
                            },
                        ]}>
                        <Checkbox> {lang.Steps.StepFour.dsgvo.split("{0}")[0]}<Typography.Link href={config.dsgvo_url} target="_blank">{lang.Steps.StepFour.dsgvoLink}</Typography.Link>{lang.Steps.StepFour.dsgvo.split("{0}")[1]}</Checkbox>

                    </Form.Item>
                    <Form.Item name="captcha" label={lang.Steps.StepFour.captcha} style={{ alignSelf: "center" }}
                        rules={[
                            {
                                validator: (_, value) =>
                                    recaptchaValue !== undefined && recaptchaValue !== null ? Promise.resolve() : Promise.reject(new Error(lang.Steps.StepFour.errorCaptcha)),
                            },
                        ]}>
                        <ReCAPTCHA
                            sitekey={process.env.REACT_APP_RECAPTCHA_SITE_KEY}
                            onChange={(value) => {

                                setRecaptchaValue(value);

                            }}

                        />
                    </Form.Item>
                </Form>
            </div>



        </Flex>

    </StepBasic></>;
}

export default StepFour;