import config from "../config";
import StepBasic from './StepBasic';

import React, { useEffect, useState, useContext } from 'react';
import { LangContext, BookingContext } from '../Context';

import { Spin, Card, Collapse, Flex, Typography } from 'antd';
const { Meta } = Card;

const ServiceCollapse = ({ myData, isLastItem }) => {

    return (
        <Collapse

            style={{
                ...(isLastItem ? {} : { marginBottom: "1rem" }),
                backgroundColor: "var(--background-3-collapse)",
                // text warp
                whiteSpace: "normal",
            }}
            items={[{ key: "serviceWidgetCollapse-" + myData.id, label: <Typography>{myData.name}</Typography>, children: <ServiceList data={myData} /> }]}
        >

        </Collapse>
    );
}

function ServiceList({ data }) {

    const lang = useContext(LangContext);
    const [bookingData, setBookingData] = useContext(BookingContext);



    return (
        <div style={{ width: "100%", height: "100%" }}>
            {data.activities.map((myData, i) => {

                const isLastItem = i === data.activities.length - 1;

                if (myData.activities !== undefined) {

                    if (myData.activities.length === 0) return null;

                    return <ServiceCollapse
                        key={"serviceWidgetCollapse" + myData.id}
                        myData={myData}
                        isLastItem={isLastItem}
                    />;
                }

                return (

                    <Card key={"serviceWidgetCard" + myData.id}
                        hoverable onClick={() => {
                            //bookingData.selectedService = { ...myData };
                            //setBookingData(bookingData);
                            setBookingData({ ...bookingData, selectedService: { ...myData } });
                        }} style={{
                            ...(isLastItem ? {} : { marginBottom: "1rem" }),
                            backgroundColor: (bookingData.selectedService?.id === myData.id ? "var(--serviceWidget-background-selected)" : "var(--serviceWidget-background)")
                        }}
                        size="default"
                    >
                        <Typography style={{ fontWeight: "bold" }}>{myData.name}</Typography>
                        <Typography style={{ opacity: 0.6 }}>{myData.description}</Typography>

                        <Flex justify='space-around' style={{ marginTop: 10 }}>
                            <Typography>{lang.service.money.replaceAll("{0}", myData.price)}</Typography>

                            <Typography>{lang.service.duration.replaceAll("{0}", myData.duration)}</Typography>
                        </Flex>
                    </Card>
                );
            })}
        </div>
    );
}


function StepOne({ screenType, serviceItems, canSubmit }) {
    const [bookingData, setBookingData] = useContext(BookingContext);
    const lang = useContext(LangContext);



    return <StepBasic
        screenType={screenType}
        storeName={serviceItems?.storeName}
        titleText={lang.Steps.StepOne.Title}
        headerColor={"var(--primary)"}
        bg={"var(--background-3)"}
        stepNumber={1}
        maxSteps={config.maxSteps}
        canSubmit={canSubmit}
        onClickNext={() => {
            bookingData.currentStep = 2;
            setBookingData({ ...bookingData });
        }}>

        {serviceItems === null ? <Flex justify="center" align="center" style={{ height: "100%" }}><Spin /></Flex> :

            serviceItems.activities.length === 0 ? <Flex justify="center" align="center" style={{ height: "100%" }}><Typography.Text type="secondary">{lang.Steps.StepOne.noServices}</Typography.Text></Flex> :

                <div style={{ width: "100%", height: "100%", textAlign: "left" }}>
                    <div style={{ paddingBottom: 15 }}>


                        <ServiceList data={serviceItems} />
                    </div>
                </div>
        }

    </StepBasic>
}

export { ServiceList };
export default StepOne;