import config from "../config";
import StepBasic from './StepBasic';

import React, { useEffect, useState, useContext } from 'react';
import { LangContext, BookingContext } from '../Context';

import { Spin, Card, Button, Collapse, Flex, Typography } from 'antd';

import dayjs, { Dayjs } from 'dayjs';

import { fetchJSON, apiPaths } from "../fetch";

/**
 * Mimic fetch with abort controller https://developer.mozilla.org/en-US/docs/Web/API/AbortController/abort
 * ⚠️ No IE11 support
 */

function TimePaper({ data }) {
    const [bookingData, setBookingData] = useContext(BookingContext);

    return <Card onClick={() => {


        setBookingData({ ...bookingData, currentStep: 4, selectedTime: data.time, chosenUser: data.user });
    }}
        size="small"
        className="timePaper"

    >
        {data.time.format("HH:mm")}
        {/*<Typography.Text style={{ marginLeft: "1rem" }}>{data.user}</Typography.Text>*/}
    </Card>
}

function StepThree({ screenType, canSubmit, isDisabled, serviceItems }) {
    const [selectedServiceAndDate, setSelectedServiceAndDate] = useState({ service: null, date: null }); //  contains the service and selected date

    const [bookingData, setBookingData] = useContext(BookingContext);
    const lang = useContext(LangContext);

    const requestAbortController = React.useRef(null);
    const [freeTimeList, setFreeTimeList] = React.useState(null);
    const [isLoading, setIsLoading] = React.useState(true);

    const fetchHighlightedDays = (date) => {
        const controller = new AbortController();

        fetchJSON(apiPaths.freeAtDay, { "date": ((bookingData.selectedDate).unix() * 1000), "activity_id": bookingData.selectedService.id, "users": bookingData.selectedUser !== null ? [bookingData.selectedUser] : [] },
            { signal: controller.signal }, { key: "stepThree" }
        )
            .then((response) => {
                console.log("freeTime", response);
                //const daysToHighlight = response.events.map((day) => dayjs(day).date());

                let events = response.events;

                let _freeTimeList = [];

                let normalFreeTimeList = [];

                const normalTimeDistance = 30; // minutes
                const maxOneTimeEvery = 15 - 1; // minutes

                for (let i = 0; i < events.length; i++) {
                    let userEvents = events[i];

                    for (let j = 0; j < userEvents.events.length; j++) {
                        let _event = userEvents.events[j];
                        let userId = userEvents.userId;

                        let start = dayjs(_event.start);
                        let end = dayjs(_event.end);

                        //if start is before today 23:59
                        if (start.isBefore(dayjs().endOf("day"))) {
                            let cmebt = 15;

                            if (serviceItems !== undefined && serviceItems.users[userId] !== undefined && serviceItems.users[userId].cmebt !== undefined) {
                                cmebt = serviceItems.users[userId].cmebt;
                                console.log("cmebt", cmebt);
                            }

                            let earliestStart = dayjs().add(cmebt, "minute");

                            if (start.isBefore(earliestStart)) {
                                start = earliestStart;


                                if (start.isAfter(end)) {
                                    continue;
                                }
                            }
                        }


                        let startCalc = dayjs(start);

                        // for loop that loops a whole day in normalTimeDistance steps
                        let startMinute = start.minute();
                        let remainder = startMinute % normalTimeDistance;

                        if (remainder !== 0) {
                            startCalc = startCalc.add(normalTimeDistance - remainder, "minute");
                        }

                        for (let k = startCalc; k < end; k = k.add(normalTimeDistance, "minute")) {
                            normalFreeTimeList.push({ time: k, user: userId });
                        }






                        _freeTimeList.push({ time: start, user: userId });
                        _freeTimeList.push({ time: end, user: userId });
                    }
                }



                // add normalFreeTimeList to _freeTimeList
                for (let i = 0; i < normalFreeTimeList.length; i++) {
                    let _event = normalFreeTimeList[i];

                    let found = false;

                    for (let j = 0; j < _freeTimeList.length; j++) {
                        let _event2 = _freeTimeList[j];

                        if (_event.user === _event2.user && _event.time.isSame(_event2.time)) {
                            found = true;
                            break;
                        }
                    }

                    if (!found) {
                        _freeTimeList.push(_event);
                    }
                }

                _freeTimeList.sort((a, b) => {
                    return a.time.unix() - b.time.unix();
                });

                console.log("_freeTimeList", _freeTimeList);


                // create a new clusterArray where when there are multiple times withhin maxOneTimeEvery minutes, they are in the same cluster
                let clusterArray = [];
                let currentCluster = [];
                let lastTime = null;
                for (let i = 0; i < _freeTimeList.length; i++) {
                    let _event = _freeTimeList[i];

                    if (lastTime === null) {
                        currentCluster.push(_event);
                    } else {
                        let diff = _event.time.diff(lastTime, "minute");

                        if (diff <= maxOneTimeEvery) {
                            currentCluster.push(_event);
                        } else {
                            clusterArray.push(currentCluster);
                            currentCluster = [];
                            currentCluster.push(_event);
                        }
                    }

                    lastTime = _event.time;
                }

                if (currentCluster.length > 0) {
                    clusterArray.push(currentCluster);
                }

                console.log("clusterArray", clusterArray);

                // filter out duplicated users and use only the smallest time for that user
                for (let i = 0; i < clusterArray.length; i++) {
                    let cluster = clusterArray[i];

                    let users = [];
                    for (let j = 0; j < cluster.length; j++) {
                        let _event = cluster[j];

                        if (users.indexOf(_event.user) === -1) {
                            users.push(_event.user);
                        }
                    }

                    let newCluster = [];
                    for (let j = 0; j < users.length; j++) {
                        let user = users[j];

                        let smallestTime = null;
                        for (let k = 0; k < cluster.length; k++) {
                            let _event = cluster[k];

                            if (_event.user === user) {
                                if (smallestTime === null || _event.time.isBefore(smallestTime)) {
                                    smallestTime = _event.time;
                                }
                            }
                        }

                        newCluster.push({ time: smallestTime, user: user });
                    }

                    clusterArray[i] = newCluster;
                }

                console.log("clusterArray2", clusterArray);

                // create a new freeTimeListFiltered where the time is a random time from the cluster
                let freeTimeListFiltered = [];
                for (let i = 0; i < clusterArray.length; i++) {
                    let cluster = clusterArray[i];

                    let randomIndex = Math.floor(Math.random() * cluster.length);

                    freeTimeListFiltered.push(cluster[randomIndex]);
                }

                setFreeTimeList(freeTimeListFiltered);
                setIsLoading(false);
            })
            .catch((error) => {
                setFreeTimeList([]);
                setIsLoading(false);

                // ignore the error if it's caused by `controller.abort`
                if (error.name !== 'AbortError') {
                    throw error;
                }
            });

        requestAbortController.current = controller;
    };

    React.useEffect(() => {
        console.log("StepThree useEffect");
        if (selectedServiceAndDate.service !== bookingData.selectedService || selectedServiceAndDate.date !== bookingData.selectedDate) {

            if (bookingData.service !== null && bookingData.selectedDate !== null) {
                fetchHighlightedDays(selectedServiceAndDate.selectedDate);
            }

            setSelectedServiceAndDate({ service: bookingData.selectedService, date: bookingData.date });

            setIsLoading(true);
        }


    }, [bookingData]);

    React.useEffect(() => {



        // abort request on unmount
        return () => requestAbortController.current?.abort();
    }, []);

    const loading = isLoading || freeTimeList === null;

    return <StepBasic
        screenType={screenType}
        storeName={serviceItems?.storeName}
        titleText={lang.Steps.StepThree.Title}
        headerColor={"var(--primary-3)"}
        bg={"var(--background)"}
        stepNumber={3}
        maxSteps={config.maxSteps}
        isDisabled={isDisabled}
        onClickBack={() => {
            bookingData.currentStep = 2;
            setBookingData({ ...bookingData });
        }}>

        {loading ? <Flex justify="center" align="center" style={{ height: "100%" }}><Spin /></Flex> : freeTimeList.length === 0 ?
            <Typography>{lang.Steps.StepThree.noFreeTime}</Typography> : <div style={{ marginBottom: 15 }}>{freeTimeList.map((data, i) => {
                return <TimePaper key={"freeTimeKey" + i} data={data} />;
            })}</div>}

    </StepBasic>;
}

export default StepThree;