import config from "../config";
import StepBasic from './StepBasic';

import React, { useEffect, useState, useContext } from 'react';
import { LangContext, BookingContext } from '../Context';

import { Spin, Button, Select, Flex, Typography } from 'antd';
import { CloseOutlined } from '@ant-design/icons';

import { DateCalendar } from '@mui/x-date-pickers/DateCalendar';
import { DayCalendarSkeleton } from '@mui/x-date-pickers/DayCalendarSkeleton';
import { LocalizationProvider } from '@mui/x-date-pickers/LocalizationProvider';
import { PickersDay } from '@mui/x-date-pickers/PickersDay';

import dayjs, { Dayjs } from 'dayjs';
import 'dayjs/locale/de';

import Badge from '@mui/material/Badge';
import { AdapterDayjs } from '@mui/x-date-pickers/AdapterDayjs';

import { fetchJSON, apiPaths } from "../fetch";

import { CheckCircleTwoTone } from '@ant-design/icons';

/**
 * Mimic fetch with abort controller https://developer.mozilla.org/en-US/docs/Web/API/AbortController/abort
 * ⚠️ No IE11 support
 */


const initialValue = dayjs(new Date());
let curMonth = initialValue;


function ServerDay(props) {
    const { highlightedDays = [], day, outsideCurrentMonth, ...other } = props;

    const isFreeTime =
        !props.outsideCurrentMonth && highlightedDays.indexOf(props.day.date()) >= 0;

    return (
        <Badge
            key={props.day.toString()}
            style={{ pointerEvents: "none", overflow: "visible" }}
            overlap="circular"
            badgeContent={isFreeTime ? <CheckCircleTwoTone twoToneColor={"#0a0"} style={{ fontSize: 16 }} /> : undefined}  >
            <PickersDay  {...other} disabled={!isFreeTime} outsideCurrentMonth={outsideCurrentMonth} day={day} style={{ backgroundColor: isFreeTime ? null : null, pointerEvents: 'auto' }} />
        </Badge>
    );
}


function StepTwo({ screenType, serviceItems, canSubmit, isDisabled }) {
    const [bookingData, setBookingData] = useContext(BookingContext);
    const lang = useContext(LangContext);

    const selectedService = bookingData.selectedService;
    const requestAbortController = React.useRef(null);
    const [highlightedDays, setHighlightedDays] = React.useState([]);
    const [isLoading, setIsLoading] = React.useState(true);



    let userOptions = [];

    if (selectedService !== null && serviceItems !== null && serviceItems.users !== undefined) {
        if (selectedService.users === undefined || Object.keys(selectedService.users).length === 0) {
            for (let userId in serviceItems.users) {
                userOptions.push({ value: userId, label: serviceItems.users[userId].name });
            }
        } else {
            for (let i = 0; i < selectedService.users.length; i++) {
                const userId = selectedService.users[i];

                let user = serviceItems.users[userId];
                let name = "Unbekannt";
                if (user)
                    name = user.name;
                userOptions.push({ value: userId, label: name });
            }
        }
    }





    const fetchHighlightedDays = (date) => {
        const controller = new AbortController();

        console.log("bookingData.selectedUser", bookingData.selectedUser)
        console.log("date", date);
        console.log("date formated de", date.format("DD.MM.YYYY"));

        fetchJSON(apiPaths.freeNextDays, { "date": (date.unix() * 1000), "activity_id": bookingData.selectedService.id, "users": (bookingData.selectedUser !== undefined && bookingData.selectedUser !== null) ? [bookingData.selectedUser] : [] }
            , { signal: controller.signal }, { key: "stepTwo" }
        )
            .then((response) => {
                console.log(response);
                const daysToHighlight = response.freeDays.map((day) => dayjs(day).date());
                setHighlightedDays(daysToHighlight);
                setIsLoading(false);
                //setSelectedService(bookingData.selectedService);
            })
            .catch((error) => {
                // ignore the error if it's caused by `controller.abort`
                if (error.name !== 'AbortError') {
                    throw error;
                }
            });

        requestAbortController.current = controller;
    };

    function reloadDays() {
        if (requestAbortController.current) {
            // make sure that you are aborting useless requests
            // because it is possible to switch between months pretty quickly
            requestAbortController.current.abort();
        }

        setHighlightedDays([]);
        const date = curMonth || bookingData.selectedDate || initialValue;
        fetchHighlightedDays(date);
        setIsLoading(true);
    }

    React.useEffect(() => {
        if (bookingData.selectedUser !== null && userOptions.filter((user) => user.value === bookingData.selectedUser).length === 0) {
            bookingData.selectedUser = null;
            setBookingData({ ...bookingData });
        }
    });

    React.useEffect(() => {
        console.log("bookingData.selectedService!", bookingData.selectedService);

        if (bookingData.selectedService !== undefined && bookingData.selectedService !== null) {
            //setSelectedService(bookingData.selectedService);
            console.log("bookingData.selectedDate", bookingData.selectedDate);
            reloadDays();
        }


    }, [bookingData.selectedService]);

    React.useEffect(() => {



        // abort request on unmount
        return () => requestAbortController.current?.abort();
    }, []);





    const handleMonthChange = (date) => {

        curMonth = date;

        reloadDays();
    };

    return <StepBasic
        screenType={screenType}
        storeName={serviceItems?.storeName}
        titleText={lang.Steps.StepTwo.Title}
        headerColor={"var(--primary-2)"}
        bg={"var(--background-2)"}
        stepNumber={2}
        currentStep={bookingData.currentStep}
        maxSteps={config.maxSteps}
        isDisabled={isDisabled}
        onClickBack={() => {
            bookingData.currentStep = 1;
            setBookingData({ ...bookingData });
        }}
        onClickNext={() => {
            bookingData.currentStep = 3;
            setBookingData({ ...bookingData });
        }}
        canSubmit={canSubmit}>


        <Flex style={{ maxWidth: 300, alignSelf: "center", width: "100%", marginBottom: 20 }}>
            <Select
                style={{ flex: 1, flexGrow: 1 }}
                value={bookingData.selectedUser}

                placeholder={lang.Steps.StepTwo.allEmployees}
                options={userOptions}
                onChange={(value) => {
                    bookingData.selectedUser = value;
                    setBookingData({ ...bookingData });
                    reloadDays();

                }}
            />
            <Button disabled={bookingData.selectedUser === null || bookingData.selectedUser === undefined}
                type="text"
                shape="circle"
                danger

                icon={<CloseOutlined />}
                style={{ marginLeft: 5 }}
                onClick={() => {
                    bookingData.selectedUser = null;
                    setBookingData({ ...bookingData });
                    reloadDays();
                }}
            />
        </Flex>


        <LocalizationProvider dateAdapter={AdapterDayjs} adapterLocale="de">
            <div style={{ display: "flex", justifyContent: "center", alignItems: "center" }}>
                <DateCalendar
                    value={bookingData.selectedDate}
                    loading={bookingData.selectedService === null || isLoading}
                    onMonthChange={handleMonthChange}
                    onYearChange={handleMonthChange}


                    minDate={dayjs()}
                    maxDate={serviceItems ? dayjs(serviceItems.maxDate) : null}

                    renderLoading={() => <DayCalendarSkeleton />}
                    onChange={(newValue) => {
                        bookingData.selectedDate = newValue;
                        setBookingData({ ...bookingData });
                    }}
                    slots={{
                        day: ServerDay,
                    }}
                    slotProps={{
                        day: {
                            highlightedDays,
                        },
                    }}
                />
            </div>
        </LocalizationProvider>

    </StepBasic>;
}

export default StepTwo;